@@ -3,9 +3,9 @@
 /**
  * @file classes/oai/ojs/OAIDAO.inc.php
  *
- * Copyright (c) 2014-2016 Simon Fraser University Library
- * Copyright (c) 2003-2016 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class OAIDAO
  * @ingroup oai_ojs
@@ -16,17 +16,16 @@
 
 import('lib.pkp.classes.oai.PKPOAIDAO');
 import('classes.issue.Issue');
+use Illuminate\Database\Capsule\Manager as Capsule;
 
 class OAIDAO extends PKPOAIDAO {
 
  	/** Helper DAOs */
  	var $journalDao;
  	var $sectionDao;
-	var $publishedArticleDao;
 	var $articleGalleyDao;
 	var $issueDao;
  	var $authorDao;
- 	var $journalSettingsDao;
 
  	var $journalCache;
 	var $sectionCache;
@@ -35,25 +34,22 @@ class OAIDAO extends PKPOAIDAO {
  	/**
 	 * Constructor.
 	 */
-	function OAIDAO() {
-		parent::PKPOAIDAO();
+	function __construct() {
+		parent::__construct();
 		$this->journalDao = DAORegistry::getDAO('JournalDAO');
 		$this->sectionDao = DAORegistry::getDAO('SectionDAO');
-		$this->publishedArticleDao = DAORegistry::getDAO('PublishedArticleDAO');
 		$this->articleGalleyDao = DAORegistry::getDAO('ArticleGalleyDAO');
 		$this->issueDao = DAORegistry::getDAO('IssueDAO');
 		$this->authorDao = DAORegistry::getDAO('AuthorDAO');
-		$this->journalSettingsDao = DAORegistry::getDAO('JournalSettingsDAO');
 
-		$this->journalCache = array();
-		$this->sectionCache = array();
+		$this->journalCache = [];
+		$this->sectionCache = [];
 	}
 
 	/**
-	 * @see lib/pkp/classes/oai/PKPOAIDAO::getEarliestDatestamp()
+	 * @copydoc PKPOAIDAO::getEarliestDatestampQuery()
 	 */
-	function getEarliestDatestamp($setIds = array()) {
-		return parent::getEarliestDatestamp('SELECT	CASE WHEN COALESCE(dot.date_deleted, a.last_modified) > i.last_modified THEN i.last_modified ELSE COALESCE(dot.date_deleted, a.last_modified) END', $setIds);
+	function getEarliestDatestampQuery() {
 	}
 
 	/**
@@ -63,7 +59,7 @@ class OAIDAO extends PKPOAIDAO {
 	 */
 	function &getJournal($journalId) {
 		if (!isset($this->journalCache[$journalId])) {
-			$this->journalCache[$journalId] =& $this->journalDao->getById($journalId);
+			$this->journalCache[$journalId] = $this->journalDao->getById($journalId);
 		}
 		return $this->journalCache[$journalId];
 	}
@@ -118,7 +114,7 @@ class OAIDAO extends PKPOAIDAO {
 			$abbrev = $journal->getPath();
 			array_push($sets, new OAISet(urlencode($abbrev), $title, ''));
 
-			$tombstoneDao = DAORegistry::getDAO('DataObjectTombstoneDAO');
+			$tombstoneDao = DAORegistry::getDAO('DataObjectTombstoneDAO'); /* @var $tombstoneDao DataObjectTombstoneDAO */
 			$articleTombstoneSets = $tombstoneDao->getSets(ASSOC_TYPE_JOURNAL, $journal->getId());
 
 			$sections = $this->sectionDao->getByJournalId($journal->getId());
@@ -172,55 +168,6 @@ class OAIDAO extends PKPOAIDAO {
 	//
 	// Protected methods.
 	//
-	/**
-	 * @see lib/pkp/classes/oai/PKPOAIDAO::getRecordSelectStatement()
-	 */
-	function getRecordSelectStatement() {
-		return 'SELECT	CASE WHEN COALESCE(dot.date_deleted, a.last_modified) < i.last_modified THEN i.last_modified ELSE COALESCE(dot.date_deleted, a.last_modified) END AS last_modified,
-			COALESCE(a.submission_id, dot.data_object_id) AS submission_id,
-			COALESCE(j.journal_id, tsoj.assoc_id) AS journal_id,
-			COALESCE(tsos.assoc_id, s.section_id) AS section_id,
-			i.issue_id,
-			dot.tombstone_id,
-			dot.set_spec,
-			dot.oai_identifier';
-	}
-
-	/**
-	 * @see lib/pkp/classes/oai/PKPOAIDAO::getRecordJoinClause()
-	 */
-	function getRecordJoinClause($articleId = null, $setIds = array(), $set = null) {
-		if (isset($setIds[1])) {
-			list($journalId, $sectionId) = $setIds;
-		} else {
-			list($journalId) = $setIds;
-		}
-		return 'LEFT JOIN published_submissions pa ON (m.i=0' . (isset($articleId) ? ' AND pa.submission_id = ?' : '') . ')
-			LEFT JOIN submissions a ON (a.submission_id = pa.submission_id' . (isset($journalId) ? ' AND a.context_id = ?' : '') . (isset($sectionId) ? ' AND a.section_id = ?' : '') .')
-			LEFT JOIN issues i ON (i.issue_id = pa.issue_id)
-			LEFT JOIN sections s ON (s.section_id = a.section_id)
-			LEFT JOIN journals j ON (j.journal_id = a.context_id)
-			LEFT JOIN data_object_tombstones dot ON (m.i = 1' . (isset($articleId) ? ' AND dot.data_object_id = ?' : '') . (isset($set) ? ' AND dot.set_spec = ?' : '') .')
-			LEFT JOIN data_object_tombstone_oai_set_objects tsoj ON ' . (isset($journalId) ? '(tsoj.tombstone_id = dot.tombstone_id AND tsoj.assoc_type = ' . ASSOC_TYPE_JOURNAL . ' AND tsoj.assoc_id = ?)' : 'tsoj.assoc_id = null') .
-			' LEFT JOIN data_object_tombstone_oai_set_objects tsos ON ' . (isset($sectionId) ? '(tsos.tombstone_id = dot.tombstone_id AND tsos.assoc_type = ' . ASSOC_TYPE_SECTION . ' AND tsos.assoc_id = ?)' : 'tsos.assoc_id = null');
-	}
-
-	/**
-	 * @see lib/pkp/classes/oai/PKPOAIDAO::getAccessibleRecordWhereClause()
-	 */
-	function getAccessibleRecordWhereClause() {
-		return 'WHERE ((s.section_id IS NOT NULL AND i.published = 1 AND j.enabled = 1 AND a.status <> ' . STATUS_DECLINED . ') OR dot.data_object_id IS NOT NULL)';
-	}
-
-	/**
-	 * @see lib/pkp/classes/oai/PKPOAIDAO::getDateRangeWhereClause()
-	 */
-	function getDateRangeWhereClause($from, $until) {
-		return (isset($from) ? ' AND CASE WHEN COALESCE(dot.date_deleted, a.last_modified) < i.last_modified THEN (i.last_modified >= ' . $this->datetimeToDB($from) . ') ELSE ((dot.date_deleted IS NOT NULL AND dot.date_deleted >= ' . $this->datetimeToDB($from) . ') OR (dot.date_deleted IS NULL AND a.last_modified >= ' . $this->datetimeToDB($from) . ')) END' : '')
-			. (isset($until) ? ' AND CASE WHEN COALESCE(dot.date_deleted, a.last_modified) < i.last_modified THEN (i.last_modified <= ' . $this->datetimeToDB($until) . ') ELSE ((dot.date_deleted IS NOT NULL AND dot.date_deleted <= ' . $this->datetimeToDB($until) . ') OR (dot.date_deleted IS NULL AND a.last_modified <= ' . $this->datetimeToDB($until) . ')) END' : '')
-			. ' ORDER BY journal_id';
-	}
-
 	/**
 	 * @see lib/pkp/classes/oai/PKPOAIDAO::setOAIData()
 	 */
@@ -233,11 +180,11 @@ class OAIDAO extends PKPOAIDAO {
 		$record->sets = array(urlencode($journal->getPath()) . ':' . urlencode($section->getLocalizedAbbrev()));
 
 		if ($isRecord) {
-			$publishedArticle = $this->publishedArticleDao->getPublishedArticleByArticleId($articleId);
+			$submission = Services::get('submission')->get($articleId);
 			$issue = $this->getIssue($row['issue_id']);
-			$galleys = $this->articleGalleyDao->getBySubmissionId($articleId)->toArray();
+			$galleys = $this->articleGalleyDao->getByPublicationId($submission->getCurrentPublication()->getId())->toArray();
 
-			$record->setData('article', $publishedArticle);
+			$record->setData('article', $submission);
 			$record->setData('journal', $journal);
 			$record->setData('section', $section);
 			$record->setData('issue', $issue);
@@ -246,6 +193,89 @@ class OAIDAO extends PKPOAIDAO {
 
 		return $record;
 	}
+
+	/**
+	 * Get a OAI records record set.
+	 * @param $setIds array Objects ids that specify an OAI set,
+	 * in hierarchical order.
+	 * @param $from int/string *nix timestamp or ISO datetime string
+	 * @param $until int/string *nix timestamp or ISO datetime string
+	 * @param $set string
+	 * @param $submissionId int optional
+	 * @param $orderBy string UNFILTERED
+	 * @return Iterable
+	 */
+	function _getRecordsRecordSet($setIds, $from, $until, $set, $submissionId = null, $orderBy = 'journal_id, submission_id') {
+		$journalId = array_shift($setIds);
+		$sectionId = array_shift($setIds);
+
+		# Exlude all journals that do not have Oai specifically turned on, see #pkp/pkp-lib#6503
+		$excludeJournals = Capsule::table('journals')
+			->whereNotIn('journal_id',function($query){
+				$query->select('journal_id')
+				->from('journal_settings')
+				->where('setting_name', 'enableOai')
+				->where('setting_value', 1);
+			})
+		->groupBy('journal_id')
+		->pluck('journal_id')
+		->all();
+
+		$params = array((int) STATUS_PUBLISHED);
+		if (isset($journalId)) $params[] = (int) $journalId;
+		if (isset($sectionId)) $params[] = (int) $sectionId;
+		if ($submissionId) $params[] = (int) $submissionId;
+		if (isset($journalId)) $params[] = (int) $journalId;
+		if (isset($sectionId)) $params[] = (int) $sectionId;
+		if (isset($set)) {
+			$params[] = $set;
+			$params[] = $set . ':%';
+		}
+		if ($submissionId) $params[] = (int) $submissionId;
+		return $this->retrieve(
+			'SELECT	GREATEST(a.last_modified, i.last_modified, p.last_modified) AS last_modified,
+				a.submission_id AS submission_id,
+				j.journal_id AS journal_id,
+				s.section_id AS section_id,
+				i.issue_id,
+				NULL AS tombstone_id,
+				NULL AS set_spec,
+				NULL AS oai_identifier
+			FROM
+				submissions a
+				JOIN publications p ON (a.current_publication_id = p.publication_id)
+				JOIN publication_settings psissue ON (psissue.publication_id = p.publication_id AND psissue.setting_name=\'issueId\' AND psissue.locale=\'\')
+				JOIN issues i ON (CAST(i.issue_id AS CHAR(20)) = psissue.setting_value)
+				JOIN sections s ON (s.section_id = p.section_id)
+				JOIN journals j ON (j.journal_id = a.context_id)
+			WHERE	i.published = 1 AND j.enabled = 1 AND a.status = ?
+				' . ($excludeJournals ?' AND j.journal_id NOT IN ('.implode(',', $excludeJournals).')':'') . '
+				' . (isset($journalId) ?' AND j.journal_id = ?':'') . '
+				' . (isset($sectionId) ?' AND p.section_id = ?':'') . '
+				' . ($from?' AND GREATEST(a.last_modified, i.last_modified, p.last_modified) >= ' . $this->datetimeToDB($from):'') . '
+				' . ($until?' AND GREATEST(a.last_modified, i.last_modified, p.last_modified) <= ' . $this->datetimeToDB($until):'') . '
+				' . ($submissionId?' AND a.submission_id = ?':'') . '
+			UNION
+			SELECT	dot.date_deleted AS last_modified,
+				dot.data_object_id AS submission_id,
+				' . (isset($journalId) ? 'tsoj.assoc_id' : 'NULL') . ' AS assoc_id,' . '
+				' . (isset($sectionId)? 'tsos.assoc_id' : 'NULL') . ' AS section_id,
+				NULL AS issue_id,
+				dot.tombstone_id,
+				dot.set_spec,
+				dot.oai_identifier
+			FROM	data_object_tombstones dot' . '
+				' . (isset($journalId) ? 'JOIN data_object_tombstone_oai_set_objects tsoj ON (tsoj.tombstone_id = dot.tombstone_id AND tsoj.assoc_type = ' . ASSOC_TYPE_JOURNAL . ' AND tsoj.assoc_id = ?)' : '') . '
+				' . (isset($sectionId)? 'JOIN data_object_tombstone_oai_set_objects tsos ON (tsos.tombstone_id = dot.tombstone_id AND tsos.assoc_type = ' . ASSOC_TYPE_SECTION . ' AND tsos.assoc_id = ?)' : '') . '
+			WHERE	1=1
+				' . (isset($set)?' AND (dot.set_spec = ? OR dot.set_spec LIKE ?)':'') . '
+				' . ($from?' AND dot.date_deleted >= ' . $this->datetimeToDB($from):'') . '
+				' . ($until?' AND dot.date_deleted <= ' . $this->datetimeToDB($until):'') . '
+				' . ($submissionId?' AND dot.data_object_id = ?':'') . '
+			ORDER BY ' . $orderBy,
+			$params
+		);
+	}
 }
 
-?>
+
