@@ -3,9 +3,9 @@
 /**
  * @file classes/db/DBResultRange.inc.php
  *
- * Copyright (c) 2014-2016 Simon Fraser University Library
- * Copyright (c) 2000-2016 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2000-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class DBResultRange
  * @ingroup db
@@ -21,13 +21,17 @@ class DBResultRange {
 	/** The number of pages to skip */
 	var $page;
 
+	/** Optional offset if pagination is not used. */
+	var $offset;
+
 	/**
 	 * Constructor.
 	 * Initialize the DBResultRange.
 	 */
-	function DBResultRange($count, $page = 1) {
+	function __construct($count, $page = 1, $offset = null) {
 		$this->count = $count;
 		$this->page = $page;
+		$this->offset = $offset;
 	}
 
 	/**
@@ -35,7 +39,8 @@ class DBResultRange {
 	 * @return boolean
 	 */
 	function isValid() {
-		return (($this->count>0) && ($this->page>=0));
+		return (($this->count > 0) && ($this->page >= 0))
+				|| ($this->count > 0 && !is_null($this->offset));
 	}
 
 	/**
@@ -69,6 +74,22 @@ class DBResultRange {
 	function setCount($count) {
 		$this->count = $count;
 	}
+
+	/**
+	 * Returns the offset of items in this range to display.
+	 * @return int
+	 */
+	function getOffset() {
+		return $this->offset;
+	}
+
+	/**
+	 * Set the offset of items in this range to display.
+	 * @param $offset int
+	 */
+	function setOffset($offset) {
+		$this->offset = $offset;
+	}
 }
 
-?>
+
