@@ -3,9 +3,9 @@
 /**
  * @file controllers/grid/admin/context/ContextGridHandler.inc.php
  *
- * Copyright (c) 2014-2019 Simon Fraser University
- * Copyright (c) 2000-2019 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2000-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class ContextGridHandler
  * @ingroup controllers_grid_admin_context
@@ -64,6 +64,8 @@ class ContextGridHandler extends GridHandler {
 			LOCALE_COMPONENT_APP_ADMIN
 		);
 
+		$this->setTitle('context.contexts');
+
 		// Grid actions.
 		$router = $request->getRouter();
 
@@ -75,8 +77,10 @@ class ContextGridHandler extends GridHandler {
 					$router->url($request, null, null, 'createContext', null, null),
 					__('admin.contexts.create'),
 					'modal_add_item',
-					true
-					),
+					true,
+					'context',
+					['editContext']
+				),
 				__('admin.contexts.create'),
 				'add_item'
 			)
@@ -102,7 +106,7 @@ class ContextGridHandler extends GridHandler {
 		// Context path.
 		$this->addColumn(
 			new GridColumn(
-				'path',
+				'urlPath',
 				'context.path',
 				null,
 				null,
@@ -181,6 +185,88 @@ class ContextGridHandler extends GridHandler {
 		return $this->editContext($args, $request);
 	}
 
+	/**
+	 * Edit an existing context.
+	 * @param $args array
+	 * @param $request PKPRequest
+	 * @return JSONMessage JSON object
+	 */
+	function editContext($args, $request) {
+		import('classes.core.Services');
+		$contextService = Services::get('context');
+		$context = null;
+
+		if ($request->getUserVar('rowId')) {
+			$context = $contextService->get((int) $request->getUserVar('rowId'));
+			if (!$context) {
+				return new JSONMessage(false);
+			}
+		}
+
+		$dispatcher = $request->getDispatcher();
+		if ($context) {
+			$apiUrl = $dispatcher->url($request, ROUTE_API, $context->getPath(), 'contexts/' . $context->getId());
+			$supportedLocales = $context->getSupportedFormLocales();
+		} else {
+			$apiUrl = $dispatcher->url($request, ROUTE_API, CONTEXT_ID_ALL, 'contexts');
+			$supportedLocales = $request->getSite()->getSupportedLocales();
+		}
+
+		$localeNames = AppLocale::getAllLocales();
+		$locales = array_map(function($localeKey) use ($localeNames) {
+			return ['key' => $localeKey, 'label' => $localeNames[$localeKey]];
+		}, $supportedLocales);
+
+		$contextForm = new \APP\components\forms\context\ContextForm($apiUrl, $locales, $request->getBaseUrl(), $context);
+		$contextFormConfig = $contextForm->getConfig();
+
+		// Pass the URL to the context settings wizard so that the AddContextForm
+		// component can redirect to it when a new context is added.
+		if (!$context) {
+			$contextFormConfig['editContextUrl'] = $request->getDispatcher()->url($request, ROUTE_PAGE, 'index', 'admin', 'wizard', '__id__');
+		}
+
+		$containerData = [
+			'components' => [
+				FORM_CONTEXT => $contextFormConfig,
+			],
+		];
+
+		$templateMgr = TemplateManager::getManager($request);
+		$templateMgr->assign([
+			'containerData' => $containerData,
+			'isAddingNewContext' => !$context,
+		]);
+
+		return new JSONMessage(true, $templateMgr->fetch('admin/editContext.tpl'));
+	}
+
+	/**
+	 * Delete a context.
+	 * @param $args array
+	 * @param $request PKPRequest
+	 * @return JSONMessage JSON object
+	 */
+	function deleteContext($args, $request) {
+
+		if (!$request->checkCSRF()) {
+			return new JSONMessage(false);
+		}
+
+		import('classes.core.Services');
+		$contextService = Services::get('context');
+
+		$context = $contextService->get((int) $request->getUserVar('rowId'));
+
+		if (!$context) {
+			return new JSONMessage(false);
+		}
+
+		$contextService->delete($context);
+
+		return DAO::getDataChangedEvent($context->getId());
+	}
+
 	/**
 	 * Display users management grid for the given context.
 	 * @param $args array
@@ -191,24 +277,6 @@ class ContextGridHandler extends GridHandler {
 		$templateMgr = TemplateManager::getManager($request);
 		$templateMgr->assign('oldUserId', (int) $request->getUserVar('oldUserId')); // for merging users.
 		parent::setupTemplate($request);
-		return $templateMgr->fetchJson('core:controllers/tab/settings/users.tpl');
-	}
-
-	//
-	// Protected helper methods.
-	//
-	/**
-	 * Return a redirect event.
-	 * @param $request Request
-	 * @param $newContextPath string
-	 * @param $openWizard boolean
-	 */
-	protected function _getRedirectEvent($request, $newContextPath, $openWizard) {
-		$dispatcher = $request->getDispatcher();
-
-		$url = $dispatcher->url($request, ROUTE_PAGE, $newContextPath, 'admin', 'contexts', null, array('openWizard' => $openWizard));
-		return $request->redirectUrlJson($url);
+		return $templateMgr->fetchJson('management/accessUsers.tpl');
 	}
 }
-
-
