@@ -156,6 +156,11 @@ class ArticleHandler extends Handler {
 					foreach ((array) $publication->getData('galleys') as $galley) {
 						if ($galley->getBestGalleyId() == $galleyId) {
 							$request->redirect(null, $request->getRequestedPage(), $request->getRequestedOp(), [$submission->getBestId()]);
+
+						// In some cases, a URL to a galley may use the ID when it should use
+						// the urlPath. Redirect to the galley's correct URL.
+						} elseif (ctype_digit($galleyId) && $galley->getId() == $galleyId) {
+							$request->redirect(null, $request->getRequestedPage(), $request->getRequestedOp(), [$submission->getBestId(), $galley->getBestGalleyId()]);
 						}
 					}
 				}
@@ -370,7 +375,7 @@ class ArticleHandler extends Handler {
 		}
 		$suppId = isset($args[1]) ? $args[1] : 0;
 		$submissionFilesIterator = Services::get('submissionFile')->getMany([
-			'submissionIds' => [$articleId->getId()],
+			'submissionIds' => [$article->getId()],
 		]);
 		foreach ($submissionFilesIterator as $submissionFile) {
 			if ($submissionFile->getData('old-supp-id') == $suppId) {
