@@ -3,9 +3,9 @@
 /**
  * @file plugins/gateways/resolver/ResolverPlugin.inc.php
  *
- * Copyright (c) 2014-2016 Simon Fraser University Library
- * Copyright (c) 2003-2016 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2014-2021 Simon Fraser University
+ * Copyright (c) 2003-2021 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class ResolverPlugin
  * @ingroup plugins_gateways_resolver
@@ -17,13 +17,10 @@ import('lib.pkp.classes.plugins.GatewayPlugin');
 
 class ResolverPlugin extends GatewayPlugin {
 	/**
-	 * Called as a plugin is registered to the registry
-	 * @param $category String Name of category plugin was registered to
-	 * @return boolean True iff plugin initialized successfully; if false,
-	 * 	the plugin will not be registered.
+	 * @copydoc Plugin::register()
 	 */
-	function register($category, $path) {
-		$success = parent::register($category, $path);
+	function register($category, $path, $mainContextId = null) {
+		$success = parent::register($category, $path, $mainContextId);
 		$this->addLocaleData();
 		return $success;
 	}
@@ -66,11 +63,10 @@ class ResolverPlugin extends GatewayPlugin {
 		switch ($scheme) {
 			case 'doi':
 				$doi = implode('/', $args);
-				$journal = $request->getJournal();
-				$publishedArticleDao = DAORegistry::getDAO('PublishedArticleDAO'); /* @var $publishedArticleDao PublishedArticleDAO */
-				$article = $publishedArticleDao->getPublishedArticleByPubId('doi', $doi, $journal?$journal->getId():null);
-				if(is_a($article, 'PublishedArticle')) {
-					$request->redirect(null, 'article', 'view', $article->getBestArticleId());
+				$submissionDao = DAORegistry::getDAO('SubmissionDAO'); /* @var $submissionDao SubmissionDAO */
+				$article = $submissionDao->getByPubId('doi', $doi, $request->getJournal());
+				if($article) {
+					$request->redirect(null, 'article', 'view', $article->getBestId());
 				}
 				break;
 			case 'vnp': // Volume, number, page
@@ -91,7 +87,7 @@ class ResolverPlugin extends GatewayPlugin {
 				$number = array_shift($args);
 				$page = (int) array_shift($args);
 
-				$issueDao = DAORegistry::getDAO('IssueDAO');
+				$issueDao = DAORegistry::getDAO('IssueDAO'); /* @var $issueDao IssueDAO */
 				$issues = $issueDao->getPublishedIssuesByNumber($journal->getId(), $volume, $number, $year);
 
 				// Ensure only one issue matched, and fetch it.
@@ -99,21 +95,23 @@ class ResolverPlugin extends GatewayPlugin {
 				if (!$issue || $issues->next()) break;
 				unset($issues);
 
-				$publishedArticleDao = DAORegistry::getDAO('PublishedArticleDAO');
-				$articles = $publishedArticleDao->getPublishedArticles($issue->getId());
-				foreach ($articles as $article) {
+				$submissionsIterator = Services::get('submission')->getMany([
+					'issueIds' => $issue->getId(),
+					'contextId' => $journal->getId()
+				]);
+				foreach ($submissionsIterator as $submission) {
 					// Look for the correct page in the list of articles.
 					$matches = null;
-					if (PKPString::regexp_match_get('/^[Pp][Pp]?[.]?[ ]?(\d+)$/', $article->getPages(), $matches)) {
+					if (PKPString::regexp_match_get('/^[Pp][Pp]?[.]?[ ]?(\d+)$/', $submission->getPages(), $matches)) {
 						$matchedPage = $matches[1];
-						if ($page == $matchedPage) $request->redirect(null, 'article', 'view', $article->getBestArticleId());
+						if ($page == $matchedPage) $request->redirect(null, 'article', 'view', $submission->getBestId());
 					}
-					if (PKPString::regexp_match_get('/^[Pp][Pp]?[.]?[ ]?(\d+)[ ]?-[ ]?([Pp][Pp]?[.]?[ ]?)?(\d+)$/', $article->getPages(), $matches)) {
+					if (PKPString::regexp_match_get('/^[Pp][Pp]?[.]?[ ]?(\d+)[ ]?-[ ]?([Pp][Pp]?[.]?[ ]?)?(\d+)$/', $submission->getPages(), $matches)) {
 						$matchedPageFrom = $matches[1];
 						$matchedPageTo = $matches[3];
-						if ($page >= $matchedPageFrom && ($page < $matchedPageTo || ($page == $matchedPageTo && $matchedPageFrom = $matchedPageTo))) $request->redirect(null, 'article', 'view', $article->getBestArticleId());
+						if ($page >= $matchedPageFrom && ($page < $matchedPageTo || ($page == $matchedPageTo && $matchedPageFrom = $matchedPageTo))) $request->redirect(null, 'article', 'view', $submission->getBestId());
 					}
-					unset($article);
+					unset($submission);
 				}
 				break;
 		}
@@ -132,10 +130,10 @@ class ResolverPlugin extends GatewayPlugin {
 	}
 
 	function exportHoldings() {
-		$journalDao = DAORegistry::getDAO('JournalDAO');
-		$issueDao = DAORegistry::getDAO('IssueDAO');
+		$journalDao = DAORegistry::getDAO('JournalDAO'); /* @var $journalDao JournalDAO */
+		$issueDao = DAORegistry::getDAO('IssueDAO'); /* @var $issueDao IssueDAO */
 		$journals = $journalDao->getAll(true);
-		$request = $this->getRequest();
+		$request = Application::get()->getRequest();
 		header('content-type: text/plain');
 		header('content-disposition: attachment; filename=holdings.txt');
 		echo "title\tissn\te_issn\tstart_date\tend_date\tembargo_months\tembargo_days\tjournal_url\tvol_start\tvol_end\tiss_start\tiss_end\n";
@@ -158,8 +156,8 @@ class ResolverPlugin extends GatewayPlugin {
 			}
 
 			echo $this->sanitize($journal->getLocalizedName()) . "\t";
-			echo $this->sanitize($journal->getSetting('printIssn')) . "\t";
-			echo $this->sanitize($journal->getSetting('onlineIssn')) . "\t";
+			echo $this->sanitize($journal->getData('printIssn')) . "\t";
+			echo $this->sanitize($journal->getData('onlineIssn')) . "\t";
 			echo $this->sanitize($startDate===null?'':strftime('%Y-%m-%d', $startDate)) . "\t"; // start_date
 			echo $this->sanitize($endDate===null?'':strftime('%Y-%m-%d', $endDate)) . "\t"; // end_date
 			echo $this->sanitize('') . "\t"; // embargo_months
@@ -174,4 +172,4 @@ class ResolverPlugin extends GatewayPlugin {
 	}
 }
 
-?>
+
