@@ -3,9 +3,9 @@
 /**
  * @file plugins/generic/usageStats/UsageStatsLoader.php
  *
- * Copyright (c) 2013-2019 Simon Fraser University
- * Copyright (c) 2003-2019 John Willinsky
- * Distributed under the GNU GPL v2. For full terms see the file docs/COPYING.
+ * Copyright (c) 2013-2020 Simon Fraser University
+ * Copyright (c) 2003-2020 John Willinsky
+ * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
  *
  * @class UsageStatsLoader
  * @ingroup plugins_generic_usageStats
@@ -245,7 +245,7 @@ class UsageStatsLoader extends FileLoader {
 	 * Get metric type on which this loader process statistics.
 	 */
 	protected function getMetricType() {
-		$application = Application::getApplication();
+		$application = Application::get();
 		$applicationName = $application->getName();
 		switch ($applicationName) {
 			case 'ojs2':
@@ -254,6 +254,9 @@ class UsageStatsLoader extends FileLoader {
 			case 'omp':
 				return OMP_METRIC_TYPE_COUNTER;
 				break;
+			case 'ops':
+				return OPS_METRIC_TYPE_COUNTER;
+				break;
 			default:
 				assert(false);
 		}
@@ -311,14 +314,13 @@ class UsageStatsLoader extends FileLoader {
 		$file = null;
 		$type = null;
 		if ($assocType == ASSOC_TYPE_SUBMISSION_FILE || $assocType == ASSOC_TYPE_SUBMISSION_FILE_COUNTER_OTHER) {
-			$submissionFileDao = DAORegistry::getDAO('SubmissionFileDAO'); /* @var $submissionFileDao SubmissionFileDAO */
-			$file = $submissionFileDao->getLatestRevision($assocId);
+			$file = Services::get('submissionFile')->get($assocId);
 		}
 
 		if ($file) $type = $this->getFileTypeFromFile($file);
 
 		if (!$type) {
-			$application = Application::getApplication();
+			$application = Application::get();
 			$applicationName = $application->getName();
 			switch ($applicationName) {
 				case 'ojs2':
@@ -356,12 +358,17 @@ class UsageStatsLoader extends FileLoader {
 	 * @return int One of the file type constants STATISTICS_FILE_TYPE...
 	 */
 	protected function getFileTypeFromFile($file) {
-		if (!is_a($file, 'PKPFile')) {
-			throw new Exception('Wrong object type, expected PKPFile.');
+		if (is_a($file, 'SubmissionFile')) {
+			$path = $file->getData('path');
+			$mimetype = $file->getData('mimetype');
+			$fileExtension = pathinfo($path, PATHINFO_EXTENSION);
+		} elseif (is_a($file, 'PKPFile')) {
+			$mimetype = $file->getFileType();
+			$fileExtension = pathinfo($file->getOriginalFileName(), PATHINFO_EXTENSION);
+		} else {
+			throw new Exception('Wrong object type. Expected SubmissionFile or PKPFile. Got ' . get_class($file));
 		}
-		$fileType = $file->getFileType();
-		$fileExtension = pathinfo($file->getOriginalFileName(), PATHINFO_EXTENSION);
-		switch ($fileType) {
+		switch ($mimetype) {
 			case 'application/pdf':
 			case 'application/x-pdf':
 			case 'text/pdf':
@@ -409,11 +416,25 @@ class UsageStatsLoader extends FileLoader {
 		$assocId = $assocTypeToReturn = null;
 		switch ($assocType) {
 			case ASSOC_TYPE_SUBMISSION:
+				$publicationId = null;
 				if (!isset($args[0])) break;
+				// If the operation is 'view' and the arguments count > 1
+				// the arguments must be: $submissionId/version/$publicationId.
+				// Else, it is not considered hier, as submission abstract count.
+				if ($op == 'view' && count($args) > 1) {
+					if ($args[1] !== 'version') break;
+					else if (count($args) != 3) break;
+					$publicationId = (int) $args[2];
+				}
 				$submissionId = $args[0];
-				$submissionDao = Application::getSubmissionDAO(); /* @var $submissionDao SubmissionDAO */
+				$submissionDao = DAORegistry::getDAO('SubmissionDAO'); /* @var $submissionDao SubmissionDAO */
 				$submission = $submissionDao->getById($submissionId);
 				if ($submission) {
+					if ($publicationId) {
+						$publicationDao = DAORegistry::getDAO('PublicationDAO'); /* @var $publicationDao PublicationDAO */
+						$publicationExists = $publicationDao->exists($publicationId, $submissionId);
+						if (!$publicationExists) break;
+					}
 					$assocId = $submission->getId();
 					$assocTypeToReturn = $assocType;
 				}
@@ -431,7 +452,7 @@ class UsageStatsLoader extends FileLoader {
 		if (!$assocId) $assocTypeToReturn = null;
 
 		if (!$assocId && !$assocTypeToReturn) {
-			$application = Application::getApplication();
+			$application = Application::get();
 			$applicationName = $application->getName();
 			switch ($applicationName) {
 				case 'ojs2':
@@ -440,6 +461,9 @@ class UsageStatsLoader extends FileLoader {
 				case 'omp':
 					list($assocId, $assocTypeToReturn) = $this->getOMPAssoc($assocType, $contextPaths, $page, $op, $args);
 					break;
+				case 'ops':
+					list($assocId, $assocTypeToReturn) = $this->getOPSAssoc($assocType, $contextPaths, $page, $op, $args);
+					break;
 			}
 		}
 		return array($assocId, $assocTypeToReturn);
@@ -461,17 +485,16 @@ class UsageStatsLoader extends FileLoader {
 			case ASSOC_TYPE_SUBMISSION_FILE:
 				if (!isset($args[0])) break;
 				$submissionId = $args[0];
-				$submissionDao = DAORegistry::getDAO('ArticleDAO');
+				$submissionDao = DAORegistry::getDAO('SubmissionDAO'); /* @var $submissionDao SubmissionDAO */
 				$article = $submissionDao->getById($submissionId);
 				if (!$article) break;
 
 				if (!isset($args[2])) break;
 				$fileId = $args[2];
-				$articleFileDao = DAORegistry::getDAO('SubmissionFileDAO');
-				$articleFile = $articleFileDao->getLatestRevision($fileId);
+				$articleFile = Services::get('submissionFile')->get($fileId);
 				if (!$articleFile) break;
 
-				$assocId = $articleFile->getFileId();
+				$assocId = $articleFile->getId();
 
 				// is the file article full text
 				$genreDao = DAORegistry::getDAO('GenreDAO');
@@ -530,21 +553,15 @@ class UsageStatsLoader extends FileLoader {
 		switch ($assocType) {
 			case ASSOC_TYPE_SUBMISSION_FILE:
 				if (!isset($args[0])) break;
-				$submissionId = $args[0];
-				$submissionDao = DAORegistry::getDAO('MonographDAO');
-				$monograph = $submissionDao->getById($submissionId);
-				if (!$monograph) break;
+				$submission = Services::get('submission')->get($args[0]);
+				if (!$submission) break;
 
 				if (!isset($args[2])) break;
-				$fileIdAndRevision = $args[2];
-				list($fileId, $revision) = array_map(function($a) {
-					return (int) $a;
-				}, preg_split('/-/', $fileIdAndRevision));
-
-				$monographFileDao = DAORegistry::getDAO('SubmissionFileDAO'); /* @var $monographFileDao SubmissionFileDAO */
-				$monographFile = $monographFileDao->getRevision($fileId, $revision);
-				if ($monographFile) {
-					$assocId = $monographFile->getFileId();
+				$fileId = $args[2];
+				$file = Services::get('submissionFile')->get($fileId);
+				if (!$file) break;
+				if ($file) {
+					$assocId = $file->getId();
 				}
 
 				$assocTypeToReturn = $assocType;
@@ -568,6 +585,46 @@ class UsageStatsLoader extends FileLoader {
 		return array($assocId, $assocTypeToReturn);
 	}
 
+	/**
+	 * Get assoc type and id from the passed page, operation and
+	 * arguments specific to OPS.
+	 * @param $assocType ASSOC_TYPE_...
+	 * @param $contextPaths array
+	 * @param $page string
+	 * @param $op string
+	 * @param $args array
+	 * @return array (assocId, assocType)
+	 */
+	protected function getOPSAssoc($assocType, $contextPaths, $page, $op, $args) {
+		$assocId = $assocTypeToReturn = null;
+		switch ($assocType) {
+			case ASSOC_TYPE_SUBMISSION_FILE:
+				if (!isset($args[0])) break;
+				$submissionId = $args[0];
+				$submissionDao = DAORegistry::getDAO('SubmissionDAO');
+				$article = $submissionDao->getById($submissionId);
+				if (!$article) break;
+
+				if (!isset($args[2])) break;
+				$fileId = $args[2];
+				$articleFile = Services::get('submissionFile')->get($fileId);
+				if (!$articleFile) break;
+
+				$assocId = $articleFile->getId();
+
+				// is the file article full text
+				$genreDao = DAORegistry::getDAO('GenreDAO');
+				$genre = $genreDao->getById($articleFile->getGenreId());
+				if ($genre->getCategory() != GENRE_CATEGORY_DOCUMENT || $genre->getSupplementary() || $genre->getDependent()) {
+					$assocTypeToReturn = ASSOC_TYPE_SUBMISSION_FILE_COUNTER_OTHER;
+				} else {
+					$assocTypeToReturn = $assocType;
+				}
+				break;
+		}
+		return array($assocId, $assocTypeToReturn);
+	}
+
 	/**
 	 * Get the context object based on the context path
 	 * array that's returned by Core::getContextPaths()
@@ -576,7 +633,7 @@ class UsageStatsLoader extends FileLoader {
 	 * @see Core::getContextPaths()
 	 */
 	protected function getContextByPath($contextPaths) {
-		$application = Application::getApplication();
+		$application = Application::get();
 		$deepestContextDepthIndex = $application->getContextDepth() - 1;
 		$contextPath = $contextPaths[$deepestContextDepthIndex];
 
@@ -600,14 +657,14 @@ class UsageStatsLoader extends FileLoader {
 				'index/index'
 			)
 		);
-		$application = Application::getApplication();
+		$application = Application::get();
 		$applicationName = $application->getName();
 		switch ($applicationName) {
 			case 'ojs2':
 				$pageAndOp = $pageAndOp + array(
 					ASSOC_TYPE_SUBMISSION_FILE => array(
 						'article/download'),
-					ASSOC_TYPE_ARTICLE => array(
+					ASSOC_TYPE_SUBMISSION => array(
 						'article/view'),
 					ASSOC_TYPE_ISSUE => array(
 						'issue/view'),
@@ -627,6 +684,15 @@ class UsageStatsLoader extends FileLoader {
 				);
 				$pageAndOp[Application::getContextAssocType()][] = 'catalog/index';
 				break;
+			case 'ops':
+				$pageAndOp = $pageAndOp + array(
+					ASSOC_TYPE_SUBMISSION_FILE => array(
+						'preprint/download'),
+					ASSOC_TYPE_SUBMISSION => array(
+						'preprint/view')
+				);
+				$pageAndOp[Application::getContextAssocType()][] = 'index';
+				break;
 		}
 		return $pageAndOp;
 	}
@@ -762,7 +828,9 @@ class UsageStatsLoader extends FileLoader {
 		$metricsDao = DAORegistry::getDAO('MetricsDAO'); /* @var $metricsDao PKPMetricsDAO */
 		$metricsDao->purgeLoadBatch($loadId);
 
-		while ($record = $statsDao->getNextByLoadId($loadId)) {
+		$records = $statsDao->getByLoadId($loadId);
+		foreach ($records as $record) {
+			$record = (array) $record;
 			$record['metric_type'] = $this->getMetricType();
 			$metricsDao->insertRecord($record);
 		}
